@@ -30,14 +30,6 @@ MBPanel *G_panel = NULL;
 
 #ifdef USE_XSETTINGS
 
-/*
-   XX = system_tray_id;
-
-/MATCHBOX/PANEL/XX/ORIENTATION north|south|east|west
-/MATCHBOX/PANEL/XX/BACKGROUND  default|rgb:XXX|pxm:XXX|trans:XXX
- Gtk/FontName
-*/
-
 #define XSET_UNKNOWN  0
 #define XSET_GTK_FONT 1
 
@@ -84,7 +76,6 @@ panel_xsettings_notify_cb (const char       *name,
                */
 	      DBG("%s() setting  XSET_GTK_FONT: '%s'\n", __func__, 
 		  setting->data.v_string);
-	      /* mb_menu_set_font (panel->mbmenu, setting->data.v_string); */
 	      mb_font_set_from_string(panel->msg_font, setting->data.v_string);
 	      
 	    }
@@ -270,6 +261,7 @@ panel_set_bg(MBPanel *panel, int bg_type, char *bg_spec)
   char *tmp_path = NULL;
 
   if (panel->bg_spec) free(panel->bg_spec);
+
   panel->bg_spec = strdup(bg_spec);
   panel->bg_type = bg_type;
 
@@ -289,7 +281,7 @@ panel_set_bg(MBPanel *panel, int bg_type, char *bg_spec)
       
       if ((img_tmp = mb_pixbuf_img_new_from_file(panel->pb, tmp_path)) == NULL)
 	{
-	  fprintf(stderr, "mbdock: failed to load %s\n", bg_spec);
+	  fprintf(stderr, "matchbox-panel: failed to load %s\n", bg_spec);
 	  panel_set_bg(panel, BG_SOLID_COLOR, DEFAULT_COLOR_SPEC);
 	  return;
 	}
@@ -406,10 +398,12 @@ panel_set_bg(MBPanel *panel, int bg_type, char *bg_spec)
 	      return;
 	    }
 
+	  /*
 	  DBG("%s() First pixel of root looks like %i, %i, %i, %i\n", 
 	      __func__, img_tmp->rgba[0], img_tmp->rgba[1],
 	      img_tmp->rgba[2], img_tmp->rgba[3]);
-	  
+	  */
+
 	  if (trans_level > 0)
 	    for (dx = 0; dx < panel->w; dx++)
 	      for (dy = 0; dy < panel->h; dy++)
@@ -585,6 +579,10 @@ panel_set_theme_from_root_prop(MBPanel *panel)
 
 		}
 
+	      if (mb_dotdesktop_get(theme, "PanelMsgFont"))
+		mb_font_set_from_string(panel->msg_font, 
+					mb_dotdesktop_get(theme, "PanelMsgFont"));
+
 	      if (mb_dotdesktop_get(theme, "PanelMsgBgCol"))
 		util_xcol_from_spec(panel, panel->msg_col, 
 				    mb_dotdesktop_get(theme, "PanelMsgBgCol"));
@@ -603,6 +601,7 @@ panel_set_theme_from_root_prop(MBPanel *panel)
 		util_xcol_from_spec(panel, panel->msg_urgent_col, 
 				    DEFAULT_MSG_BGURGCOL);
 
+
 	      if (mb_dotdesktop_get(theme, "PanelMsgFgCol"))
 		util_xcol_from_spec(panel, panel->msg_fg_col, 
 				    mb_dotdesktop_get(theme, 
@@ -661,10 +660,11 @@ panel_usage(char *bin_name)
 	  "--id                 <int> Panel ID\n"
           "--size, -s           <int> width/height of dock in pixels\n"
           "--orientation        <north|east|south|west>\n"
-          "--default-apps, -da <app list> comma seperated list of apps to\n"
+          "--default-apps, -da  <app list> comma seperated list of apps to\n"
           "                     add to a tray when no session exists\n"
-          "--margin-start       <+int> initial app offset in pixels (panel start)\n"
-          "--margin-end         <+int> initial app offset in pixels (panel end)\n"
+          "--margins            <int>[,<int>]  specify left+right[, top+bottom ] panel\n"
+	  "                     margins in pixels ( default: 2,2 )\n"
+	  "--padding            Specify padding betweeen applets in pixels ( default: 2)\n"
 	  "--titlebar           Request panel in titlebar - see docs for limitations\n"
           "--no-session, -ns    No session saving.\n"
           "--no-menu, -nm       No popup menu\n"   
@@ -1169,9 +1169,11 @@ panel_main(MBPanel *panel)
 		      if (!(panel->use_flip && had_rotation))
 			{
 			  char *tmp_str = NULL;
-			  if (panel->bg_spec) 
-			    tmp_str = strdup(panel->bg_spec) ;
-			  panel_set_bg(panel, panel->bg_type, tmp_str);
+			  if (panel->bg_spec)
+			    {
+			      tmp_str = strdup(panel->bg_spec) ;
+			      panel_set_bg(panel, panel->bg_type, tmp_str);
+			    }
 			  if (tmp_str) free(tmp_str);
 			}
 
@@ -1460,7 +1462,7 @@ panel_reorder_apps(MBPanel *panel)
 
   papp = panel->apps_start_head;
 
-  offset = panel->margin_start;
+  offset = panel->margin_sides;
 
   DBG("%s() start list:\n", __func__ );
 
@@ -1491,7 +1493,7 @@ panel_reorder_apps(MBPanel *panel)
 
   if (papp)
     {
-      offset = ( PANEL_IS_VERTICAL(panel) ? panel->h : panel->w ) - panel->margin_end;
+      offset = ( PANEL_IS_VERTICAL(panel) ? panel->h : panel->w ) - panel->margin_sides;
       
       while( papp != NULL)
 	{
@@ -1523,11 +1525,12 @@ panel_reorder_apps(MBPanel *panel)
 MBPanel 
 *panel_init(int argc, char *argv[])
 {
-  int panel_length;
-  XGCValues gv;
+  int                  panel_length;
+  XGCValues            gv;
   XSetWindowAttributes dattr;
-  unsigned long dattr_flags = CWBackPixel;
-  XSizeHints size_hints;
+  unsigned long        dattr_flags = CWBackPixel;
+  XSizeHints           size_hints;
+  XWMHints            *wm_hints;
 
   unsigned long wm_struct_vals[4];
 
@@ -1588,8 +1591,8 @@ MBPanel
   panel->apps_start_head     = NULL;
   panel->apps_end_head       = NULL;
   panel->ignore_next_config  = False;
-  panel->margin_start        = 2;
-  panel->margin_end          = 2;
+  panel->margin_sides        = 2;
+  panel->margin_topbottom    = 2;
   panel->bg_spec             = NULL;
   panel->want_titlebar_dest  = False;
 
@@ -1597,6 +1600,7 @@ MBPanel
     if (!strcmp ("-display", argv[i]) || !strcmp ("-d", argv[i])) {
       if (++i>=argc) panel_usage (argv[0]);
       display_name = argv[i];
+      setenv("DISPLAY", display_name, 1);
       continue;
     }
     if (!strcmp ("--no-session", argv[i]) || !strcmp ("-ns", argv[i])) {
@@ -1636,7 +1640,7 @@ MBPanel
     }
     if (strstr (argv[i], "-geometry") || !strcmp ("-g", argv[i])) {
       if (++i>=argc) panel_usage (argv[0]);
-      fprintf(stderr,"mbdock: -geometry is depreciated, please consider --size  and --orientation instead\n");
+      fprintf(stderr,"matchbox-panel: -geometry is depreciated, please consider --size  and --orientation instead\n");
       geometry_str = argv[i];
       continue;
     }
@@ -1646,16 +1650,34 @@ MBPanel
       if (panel->default_panel_size < 1) panel_usage (argv[0]);
       continue;
     }
-    if (strstr (argv[i], "--margin-start")) {
+    if (strstr (argv[i], "--padding") || !strcmp ("-s", argv[i])) {
       if (++i>=argc) panel_usage (argv[0]);
-      panel->margin_start = atoi(argv[i]);
-      if (panel->margin_start < 1) panel_usage (argv[0]);
+      panel->padding = atoi(argv[i]);
+      if (panel->padding < 0) panel_usage (argv[0]);
       continue;
     }
-    if (strstr (argv[i], "--margin-end")) {
-      if (++i>=argc) panel_usage (argv[0]);
-      panel->margin_end = atoi(argv[i]);
-      if (panel->margin_end < 1) panel_usage (argv[0]);
+    if (strstr (argv[i], "--margins")) {
+      char *p = NULL;
+
+      if (++i>=argc) 
+	panel_usage (argv[0]);
+
+      p = argv[i];
+
+      if ((p = strchr(argv[i], ',')) != NULL)
+	{
+	  *p = '\0'; p++;
+	}
+
+      panel->margin_sides = atoi(argv[i]);
+      if (panel->margin_sides < 0) panel_usage (argv[0]);
+
+      if (p != NULL)
+	{
+	  panel->margin_topbottom = atoi(p);
+	  if (panel->margin_topbottom < 0) panel_usage (argv[0]);
+	}
+
       continue;
     }
     if (!strcmp ("--bgpixmap", argv[i]) || !strcmp ("-b", argv[i])) {
@@ -1674,11 +1696,6 @@ MBPanel
       continue;
 
     }
-    if (!strcmp ("--border", argv[i])) {
-      if (++i>=argc) panel_usage (argv[0]);
-      panel_border_sz = atoi(argv[i]);
-      continue;
-    }
     if (!strcmp ("--orientation", argv[i])) {
       Bool found = False;
       if (++i>=argc) panel_usage (argv[0]);
@@ -1703,6 +1720,10 @@ MBPanel
       exit(1);
     }
 
+   if (getenv("MB_SYNC")) 
+     XSynchronize (panel->dpy, True);
+
+
   panel->screen = DefaultScreen(panel->dpy);
   panel->win_root = RootWindow(panel->dpy, panel->screen);
 
@@ -1740,15 +1761,6 @@ MBPanel
   else
       panel_length = panel->w;
 
-  /* Clip margin values to 40% */
-  if (panel->margin_start > ( ( panel_length * 40 ) / 100 ) 
-      || panel->margin_end > ( ( panel_length * 40 ) / 100  ))
-    {
-      fprintf(stderr, "mbdock: Panel margins too large. clipping.\n");
-      panel->margin_start = 2;
-      panel->margin_end   = 2;
-    }
-
   if (geometry_str)
     XParseGeometry(geometry_str, &panel->x, &panel->y, &panel->w, &panel->h);  
 
@@ -1816,6 +1828,9 @@ MBPanel
   panel->atoms[ATOM_NET_WM_STATE_TITLEBAR]
     = XInternAtom(panel->dpy, "_MB_WM_STATE_DOCK_TITLEBAR", False);
 
+  panel->atoms[ATOM_MB_DOCK_TITLEBAR_SHOW_ON_DESKTOP]
+    = XInternAtom(panel->dpy, "_MB_DOCK_TITLEBAR_SHOW_ON_DESKTOP", False);
+
   panel->atoms[ATOM_MB_SYSTEM_TRAY_CONTEXT]
     = XInternAtom(panel->dpy, "_MB_SYSTEM_TRAY_CONTEXT", False);
 
@@ -1873,28 +1888,40 @@ MBPanel
   XSetStandardProperties(panel->dpy, panel->win, win_name, 
 			 win_name, 0, argv, argc, &size_hints);
 
-  /*
-    XXX For now the menu popup button is removed.
-        If goes down well, need to rip old code out. 
-  
-  panel_menu_button_create(panel);
-
-  */
-
   XChangeProperty(panel->dpy, panel->win, 
 		  panel->atoms[ATOM_WM_WINDOW_TYPE], XA_ATOM, 32, 
 		  PropModeReplace,
 		  (unsigned char *) &panel->atoms[ATOM_WM_WINDOW_TYPE_DOCK], 
 		  1);
 
+  wm_hints = XAllocWMHints();
+  wm_hints->input = False;
+  wm_hints->flags = InputHint;
+  XSetWMHints(panel->dpy, panel->win, wm_hints );
+  XFree(wm_hints);
+
   if (panel->want_titlebar_dest)
     {
+      Atom state_list[] 
+	= { panel->atoms[ATOM_NET_WM_STATE_TITLEBAR],
+	    panel->atoms[ATOM_MB_DOCK_TITLEBAR_SHOW_ON_DESKTOP] };
+
+      int n_states = 1;
+
+      if (getenv("MB_PANEL_NO_DESKTOP_HIDE")) n_states = 2;
+
       panel->use_flip = False;
 
+      /* 
+       *   We need someway ( selection ? ) to check if the 	 
+       *   wm is running and only map then - not before 
+       *   
+       */
+
       XChangeProperty(panel->dpy, panel->win, 
 		      panel->atoms[ATOM_NET_WM_STATE], XA_ATOM, 32, 
 		      PropModeReplace, 
-		      (unsigned char *) &panel->atoms[ATOM_NET_WM_STATE_TITLEBAR], 1);
+		      (unsigned char *) &state_list, n_states);
 
       panel->orientation = North;
     }
@@ -2010,7 +2037,7 @@ int main(int argc, char *argv[])
       XSetSelectionOwner(panel->dpy, panel->atoms[ATOM_SYSTEM_TRAY],
 			 panel->win, CurrentTime);
     } else {
-      fprintf(stderr, "Panel already exists. aborting. Try running mbdock with the --id switch.\n"); 
+      fprintf(stderr, "Panel already exists. aborting. Try running matchbox-panel with the --id switch.\n"); 
       exit(0);
     }
 
@@ -2021,6 +2048,8 @@ int main(int argc, char *argv[])
 
   XMapWindow (panel->dpy, panel->win);
 
+  XSync(panel->dpy, False);
+
   session_init (panel);
 
   panel_main(panel);
